/**********************************************\
|                                              |
| ConsoleControl                               |
|                                              |
| Author: Sean Tasker                          |
|         slip@ice-d.com                       |
|         ice development                      |
|         www.ice-d.com                        |
|                                              |
| Date: 30-03-06                               |
|                                              |
| Description:                                 |
|   ConsoleControl is a virtual console class. |
|   It's designed to give control over where   |
|   characters will be displayed. It does this |
|   by buffering the contents of the console   |
|   and displaying all at once.                |
|                                              |
|   Refer to ConsoleControl.h for more info    |
|                                              |
\**********************************************/

#include "ConsoleControl.h"
using namespace std;

ConsoleControl::ConsoleControl(int bufWidth, int bufHeight)
{
	cp=cx=cy=0;
	bufferCreateError=false;		//No error yet
	if(bufWidth)
		bufferWidth=bufWidth;
	else
		bufferWidth=DEFAULT_BUFFER_WIDTH;
	if(bufHeight)
		bufferHeight=bufHeight;
	else
		bufferHeight=DEFAULT_BUFFER_HEIGHT;
	bufferSize=bufferWidth*bufferHeight;
	buffer=0;
	buffer=new char[bufferSize];
	if(!buffer)						//WTF? out of memory??
		bufferCreateError=true;
	else
		ClearBuffer();
}

ConsoleControl::~ConsoleControl()
{
	if(buffer)				//Bye bye buffer
		delete buffer;
}

int ConsoleControl::GetSize() const
{
	return bufferSize;
}

int ConsoleControl::GetWidth() const
{
	return bufferWidth;
}

int ConsoleControl::GetHeight() const
{
	return bufferHeight;
}

int ConsoleControl::GetCursorPosition() const
{
	return cp;
}

int ConsoleControl::GetCursorPositionX() const
{
	return cx;
}

int ConsoleControl::GetCursorPositionY() const
{
	return cy;
}

char *ConsoleControl::GetCol(int col) const		//TO DO: Setup a type that has the width of the buffer in data?
												//       Copy contents to a buffer sizeof(bufferHeight); return buffer
{
	return 0;
}

char *ConsoleControl::GetRow(int row) const					//Address is direct to buffer not just copy of contents
{
	return ((char*)&(buffer[row*bufferWidth]));
}

char ConsoleControl::GetChar(int pos) const
{
	return (buffer[pos]);
}

char ConsoleControl::GetChar(int x, int y) const
{
	return (buffer[x+(y*bufferWidth)]);
}

void ConsoleControl::MoveCursor(int x, int y)
{
	MoveCursor(x+(y*bufferWidth));
}

void ConsoleControl::MoveCursor(int pos)
{
	cp=pos;
	if(cp>=bufferSize)
		cp=0;
	else
		if(cp<=0)
			cp=bufferSize-1;

}

void ConsoleControl::printf(const char* msg, ...)
{
	va_list arglist;
	va_start(arglist, msg);
	char * tempBuff=new char[bufferSize];
	vsprintf_s(tempBuff,bufferSize,msg,arglist);					//Use for Visual Studio 2005+
	//vsprintf(tempBuff,msg,arglist);									//Use for Visual Studio 2003
	print(tempBuff);
	va_end(arglist);
}

void ConsoleControl::print(char*msg)			//Because it was before printf
{
	for(int i=0;msg[i]!='\0';++i)
	{
		buffer[cp]=msg[i];
		MoveCp();
	}
}

void ConsoleControl::print(char c)				//Because it was before printf
{
	buffer[cp]=c;
	MoveCp();
}

void ConsoleControl::Display()
{
	char c;
	for(int i=0; i<bufferSize;++i)			//Hack
	{
		c=buffer[i];
		cout << c;
	}
}

void ConsoleControl::DisplaySeg(int start, int finish)
{
	char c;
	if(finish>=bufferSize)
		finish=bufferSize;
	for(int i=start; i<finish;++i)			//Hack
	{
		c=buffer[i];
		cout << c;
	}
}

void ConsoleControl::ClearBuffer()
{
	//Clear the buffer with Spaces
	for(int i=0; i<bufferSize;++i)
		buffer[i]=' ';
}

void ConsoleControl::MoveCp()
{
	cp++;
	if(cp>=bufferSize)
		cp=0;
	cx=cp-(cy*bufferWidth);
	cy=(int)(cp/bufferWidth);
}

char ConsoleControl::SingleCharInput(int x, int y)				//MODIFY
{
	char c=0;
	//MoveCursor(0,0);
	DisplaySeg(0,x+(y*bufferWidth));
	while(!(c=getchar())){}									//Nope needs work -> cin >> aString;
	DisplaySeg(x+(y*bufferWidth),bufferSize);
	return c;
}

void ConsoleControl::MoveCursorVert(int a)
{
	MoveCursor(cx,cy+a);
}

void ConsoleControl::ClearRow(int row)
{
	if(row>=bufferHeight)
		return;
	row=(row*bufferWidth);
	for(int c=0;c<bufferWidth;c++)
		buffer[c+row]=' ';
}

void ConsoleControl::ClearColum(int col)
{
	if(col>=bufferWidth)
		return;
	for(int c=col;c<bufferSize;c+=bufferWidth)
		buffer[c]=' ';
}

void ConsoleControl::EnsureAlignment()
{
	cout << endl;
}
